'************************************************************************
'GL2_69PIC.BAS Graphic application for use with a T6963C (or compatible)
'graphic LCD controller connected to a Microchip(TM) PIC 16F877
'Written in MELabs PicBasicPro(TM) Ver 2.40
'Version 0.9.8
'SHARP/SANYO LCD used: 240x64 graphic LCD
'COMPSys GLI2 Development board
'Use with GLIPIC3 v0.9.6
'BMP images can be made with MSPaint (B/W 1 bit resolution 240x64 pixels)
'================== NOT FOR COMMERICIAL USE =============================
'
'       Author: Ranjit Diol - COMPSys, June 2001
'            (c) Copyright Ranjit Diol, June 2001
'
'   DISCLAIMER: This file is being released as non-commericial
'   freeware. It is being provided "AS IS", neither the author, 
'   nor COMPSys shall be held liable for any damages direct or indirect 
'   caused by its use.
'
'   LICENSE: This application may be used in its entirety or 
'   in parts within some other non-commercial application as long
'   as the author is given credit for those parts used.
'
'        Contact: rsdiol@compsys1.com for more information
'
'**************************************************************************

DEFINE	LOADER_USED	1	'If using ISP

DEFINE OSC 20			'Adjust to suit your design
ADCON1	= 7			'Adjust ADC for your needs

'************************
' LCD Module constants  *
'************************
'
'Register set commands:
CURSOR_PTR_SET          con     $21   'Cursor Pointer Set
OFFSET_REG_SET          con     $22   'Offset Register Set
ADDR_PTR_SET            con     $24   'Address Pointer Set

'Control Word Set commands:
TEXT_HOME_SET           con     $40   'Text Home Address Set
TEXT_AREA_SET           con     $41   'Text Area Set
GRAPH_HOME_SET          con     $42   'Graphics Home address Set
GRAPH_AREA_SET          con     $43   'Graphics Area Set

'Mode Set commands (OR with CG rom commands):
OR_MODE                 con     $80   'OR mode
XOR_MODE                con     $81   'XOR mode
AND_MODE                con     $83   'AND mode
TEXT_ATTR_MODE          con     $84   'Text Attribute mode
INT_CG_MODE             con     $80   'Internal CG ROM mode
EXT_CG_MODE             con     $88   'External CG ROM mode

'Display Mode commands (OR together required bits):
DISPLAY_OFF             con     $90   'display off
BLINK_ON                con     $91
CURSOR_ON               con     $92
TEXT_ON                 con     $94
GRAPHIC_ON              con     $98
TEXT_AND_GRAPH_ON	con     $9C

'Cursor Pattern Select
CURSOR_1LINE            con     $A0
CURSOR_2LINE            con     $A1
CURSOR_3LINE            con     $A2
CURSOR_4LINE            con     $A3
CURSOR_5LINE            con     $A4
CURSOR_6LINE            con     $A5
CURSOR_7LINE            con     $A6
CURSOR_8LINE            con     $A7

'Data Auto Read/Write
DATA_AUTO_WR            con     $B0
DATA_AUTO_RD            con     $B1
AUTO_DATA_RESET         con     $B2

'Data Read/Write
DATA_WR_INC             con     $C0   'Data write and increment addr
DATA_RD_INC             con     $C1   'Data read and increment addr
DATA_WR_DEC             con     $C2   'Data write and decrement addr
DATA_RD_DEC             con     $C3   'Data read and decrement addr
DATA_WR                 con     $C4   'Data write - no addr change
DATA_RD                 con     $C5   'Data read - no addr change

'Screen Peek
SCREEN_PEEK             con     $E0

'Screen Copy
SCREEN_COPY             con     $E8

'Bit Set/Reset (OR with bit number 0-7)

BIT_RESET               con     $F0
BIT_SET                 con     $F8

'************************
' STATUS bits           *
'************************

STA0                    con     0       'command excution status
STA1                    con     1       'data read/write status
STA2                    con     2       'auto data read status
STA3                    con     3       'auto data write status
STA5                    con     5       'controller status
STA6                    con     6       'peek/copy error flag
STA7                    con     7       'blink? or blank? status

ASCII_OFFSET		con     $20     'ASCII offset for conversion
GL_PAUSE		con     2
GL_ROW			con     32
GL_COL			con     64
GL_XMAX			con     239
GL_XMIN			con     0
GL_YMAX			con     63
GL_YMIN                 con     0
GL_LINES		con	8

'Variables used
'PLEASE NOTE: that this application is taken
'from another larger app and may contain variables, constants
'and routines that are never used.

'Serial I/O definitions

out_pin	 var	PORTC.6		'Serial OUT pin
in_pin	 var    PORTC.7		'Serial IN pin
ser_baud con	32		'19200 Baud

in_dat   var    BYTE		'In coming byte data
in_ser   var    BYTE[15]        'In coming data buffer
in_lsb   var    BYTE            'In coming lsb of word
in_msb   var    BYTE            'In coming msb of word
gl_chr   var    BYTE            'Character byte
eep_addr var    WORD		'External eeprom address

hex_nibble var  BYTE		'Var used for incoming data

'I2C EEprom
scl	 VAR 	PORTC.3		'I2C SCL  for ext eeprom 
sda	 VAR	PORTC.4		'I2C SDA for ext eeprom
ctl      CON    $A0		'EEprom control code

'Variables used
'GL1 settings
' *** MODIFY AS REQUIRED FOR OTHER PINS AND PORTS *****
'
'glcd_tris 	var 	TRISD		'Data bus direction
'glcd_dat	var 	PORTD		'LCD 8 bit data bus
'glcd_wr	 	var	PORTB.1		'Write control bit
'glcd_rd	 	var 	PORTB.2		'Read control bit
'glcd_ce	 	var 	PORTC.1		'Chip enable control bit
'glcd_cd	 	var 	PORTC.2		'Command and Data toggle bit
'glcd_rst	var	PORTC.0		'Reset control bit
'glcd_fs	 	var	PORTC.5		'Font select control bit
''glcd_bl	 	var	PORTA.5		'Back light control bit
'glcd_bsy	var	PORTD.7         'In case there is a need, not really used
''glcd_btn	var     PORTB.0		'A button for debugging purposes
'glcd_sta0	var     PORTD.0		'Status Bit 0	
'glcd_sta1	var     PORTD.1		'Status Bit 1
'glcd_sta2       var     PORTD.2		'Status Bit 2
'glcd_sta3       var     PORTD.3		'Status Bit 3

'Set appropriate port/bit dir
'TRISB.0  =      0
'TRISB.1  =      0
'TRISB.2  =      0
'TRISD	 = 	0
'TRISC.1	 =	0
'TRISC.0	 =	0
'TRISC.2	 =	0
'TRISC.5	 =	0
'TRISA.4	 =	0
'TRISA.5	 =	0

' *** GL2 Settings *****
'
glcd_tris 	var 	TRISD		'Data bus direction
glcd_dat	var 	PORTD		'LCD 8 bit data bus
glcd_wr	 	var  	PORTC.0		'Write control bit
glcd_rd	 	var 	PORTC.1		'Read control bit
glcd_ce	 	var 	PORTC.2		'Chip enable control bit
glcd_cd	 	var 	PORTC.5		'Command and Data toggle bit
glcd_rst	var 	PORTE.2		'Reset control bit
glcd_fs 	var 	PORTE.1		'Font select control bit
glcd_bsy	var 	PORTD.7         'In case there is a need, not really used
glcd_sta0	var     PORTD.0		'Status Bit 0	
glcd_sta1	var     PORTD.1		'Status Bit 1
glcd_sta2       var     PORTD.2		'Status Bit 2
glcd_sta3       var     PORTD.3		'Status Bit 3

'Set appropriate port/bit dir
TRISD   =       0
TRISC.0  =      0
TRISC.1  =      0
TRISC.2  =      0
TRISC.5	 =	0
TRISE.1	 =	0
TRISE.2	 =	0


'Terminal variables
in_array var    BYTE[32]		'In coming array
in_ctl	 var    BYTE[8]			'In coming control array
buff_count	var	BYTE		'General counter
gl_index	var	BYTE		'Index counter
gl_delay	var     BYTE		'Delay variable
gl_idx		var     BYTE		'General use
gl_idy		var     BYTE            'General use


'General use variables used
glcd_msb	var	BYTE		'Data 1 variable
glcd_lsb	var	BYTE 		'Data 2 variable
glcd_cmd	var	BYTE 		'Command variable
gl_tmp	 	var	BYTE		'Temp var		
gl_old	 	var     BYTE		'Previous byte var
gl_x     	var     WORD		'16 bit var for address loop
gl_y	 	var     WORD		'16 bit var for address loop
gl_i	 	var	BYTE
gl_j	 	var     BYTE
gl_k	 	var     BYTE
gl_l	 	var     BYTE
gl_m	 	var     BYTE
num_1	 	var	BYTE
num_2	 	var	BYTE
gl_stp	 	var	BYTE		'Step var in FOR..NEXT
gl_ad1	 	var	BYTE		'Addr var
gl_ad2	 	var	BYTE		'Addr var
graph_home 	var 	WORD		'LCD graphic start addr
graph_area 	var 	WORD		'LCD graphic bytes per line
text_home  	var 	WORD		'LCD text start addr
text_area  	var 	WORD		'LCD text chars per line
gl_xpos		var     BYTE
gl_ypos         var     BYTE
gl_addr         var     WORD
lcd_cmd		var     BYTE
rs_flag		var     BYTE		'Reset pixel flag
gl_byte		var	BYTE		'Graphic byte
gl_char		var     BYTE		'Text byte
ptr_flag	var	BYTE		'Addr ptr flag
gl_oldaddr	var	WORD		'Var to keep track of address
gl_oldx		var     BYTE		'Variables to keep track
gl_oldy		var     BYTE		'of x and y values


'Initialize variables
gl_tmp   =  0
gl_i     =  0
gl_stp   =  1
rs_flag  =  0
ptr_flag =  DATA_WR



'Line variables
gl_x1			var	BYTE
gl_x2			var	BYTE
gl_y1			var	BYTE
gl_y2			var	BYTE
gl_inc1			var	WORD
gl_inc2			var	WORD

'FS offsets
font_size		var     BYTE
font_byte		var     BYTE



'Initial settings of port and bits
glcd_dat = 0	'Clear PORTD
glcd_wr  = 1    'Write high 
glcd_rd  = 1	'Read high
glcd_ce  = 1  	'Chip enable high (active low)
glcd_cd  = 1  	'Command high
glcd_fs  = 0	'Set Font size (1=6x8 and 0=8x8 font)
glcd_rst = 1 	'RESET always high (low will reset the LCD controller)
'glcd_bl  = 0	'Backlight 0=off 1=on
'glcd_btn = 0 	'debug led
pause 500	'Let things stabilize

'Initialize LCD VRAM area variables


graph_home      =     $1000    'Decimal 4096
text_home		=     $0000
if glcd_fs = 0 then
	font_size = 8
	font_byte = 7
	graph_area		=  $0020    '32 bytes per line
	text_area       =  $0020    '32 8x8 chars per line
   else
	font_size = 6
	font_byte = 5
	graph_area		=     $0028    '40 bytes per line
	text_area               =     $0028    '40 6x8 chars per line
endif

glcd_rst = 0	' Reset after power-on
pause 1
glcd_rst = 1

goto main '****** jump past the routines below ******


'*********** PBP Subroutines used in this program *******

send_2:
	gosub busy_chk
	glcd_dat = glcd_lsb
	glcd_cd  = 0
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
send_1:
	gosub busy_chk
	glcd_dat = glcd_msb
	glcd_cd  = 0
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
send_cmd:
	gosub busy_chk
	glcd_dat = glcd_cmd
	glcd_rd  = 1
	glcd_ce  = 0
	glcd_wr  = 0
	'pauseus GL_PAUSE
	glcd_wr  = 1
	glcd_ce  = 1
return


busy_chk:
	glcd_ce = 1
	glcd_tris= %11111111
	glcd_cd = 1
	glcd_wr = 1
	glcd_ce = 0
	glcd_rd = 0
	'pauseus GL_PAUSE
	if (glcd_sta0 <> 1) AND (glcd_sta1 <> 1) then busy_chk
	glcd_rd = 1
        glcd_ce = 1
	glcd_tris = %00000000
return

read1:
	gosub busy_chk
	glcd_tris = $ff
        glcd_cd  = 0
        glcd_wr  = 1
        glcd_rd  = 0
	glcd_ce  = 0
	pauseus GL_PAUSE
	gl_chr = glcd_dat
        glcd_ce = 1
	glcd_rd = 1
	glcd_tris = $00
return

read_dat:
'Assumes gl_x and gl_y and ptr_flag are given. Returns gl_chr
	gosub gl_gaddr
	glcd_cmd = ptr_flag
	gosub send_cmd
	gosub read1
return

'Set address given x,y coordinates
gl_gaddr:
	gl_addr	= graph_home + (gl_y * GL_ROW) + (gl_x/font_size)
	gl_oldaddr = gl_addr
	glcd_msb = gl_addr.BYTE1
	glcd_lsb = gl_addr.BYTE0
	glcd_cmd = ADDR_PTR_SET
	gosub send_2
return



'Initialize and clear display
gl_init:				'Initialization

	'See data sheets for more info for the commands and parameters
        
	' Set Graphic Home Addr
	glcd_msb = graph_home.BYTE1		
	glcd_lsb = graph_home.BYTE0
	glcd_cmd = GRAPH_HOME_SET
        gosub send_2

	' Set Graphic Area
	glcd_msb = graph_area.BYTE1
	glcd_lsb = graph_area.BYTE0
	glcd_cmd = GRAPH_AREA_SET
        gosub send_2

        ' Set Text Home Addr
	glcd_msb = text_home.BYTE1
	glcd_lsb = text_home.BYTE0
	glcd_cmd = TEXT_HOME_SET
        gosub send_2

	' Set Text Area
	glcd_msb = text_area.BYTE1
	glcd_lsb = text_area.BYTE0
	glcd_cmd = TEXT_AREA_SET
        gosub send_2

	' Set OR mode
	glcd_cmd = OR_MODE
	gosub send_cmd
'End of Init routine


grf_clr:
	' Set Addr Ptr
	glcd_msb = $00
	glcd_lsb = $00
	glcd_cmd = ADDR_PTR_SET
	GoSub send_2

	' Set Graphic mode on	
	glcd_cmd = GRAPHIC_ON
	GoSub send_cmd

    For gl_i = 1 to 240			'Clear Graphic Screen of a 240x64 screen
	For gl_j = 1 to 64
	  glcd_msb = $00			'Clear byte
	  glcd_cmd = DATA_WR_INC	'Command
	  GoSub send_1				'Send glcd_msb followed by glcd_cmd
	Next
    Next						'Do it 240x64 times
	' Set Graphic mode on	
	glcd_cmd = TEXT_AND_GRAPH_ON
	GoSub send_cmd
Return


gl_txt_pos:
	gl_addr = (text_home + (gl_y/8 * GL_ROW) + gl_x/font_size)
	glcd_msb = gl_addr.BYTE1
	glcd_lsb = gl_addr.BYTE0
	glcd_cmd = ADDR_PTR_SET
	gosub send_2
return


put_char:			'Assumes gl_char,gl_x,gl_y and ptr_flag
	gl_char = gl_char - $20
	gosub txt_on	'Text mode to place char
	gosub gl_txt_pos	'Set text addr given gl_x,gl_y
	glcd_msb = gl_char
	glcd_cmd = ptr_flag
	gosub send_1
	gosub grf_txt_on
return


put_byte:		      'Assumes gl_x,gl_y and gl_byte and ptr_flag
    gosub grf_on	      'Graphic mode
	gosub gl_gaddr        'Set address
	glcd_msb = gl_byte
	glcd_cmd = ptr_flag
	gosub send_1
return


grf_on:
	glcd_cmd = GRAPHIC_ON
	gosub send_cmd
return

txt_on:
	glcd_cmd = TEXT_ON
	gosub send_cmd
return

grf_txt_on:
	glcd_cmd = TEXT_AND_GRAPH_ON
	gosub send_cmd
return


'***** Convert incoming ascii to hex byte Routine *****
gl_find:
Lookdown in_array[gl_index],["0123456789abcdef"],hex_nibble
return	

grf_home:	
	' Set Addr Ptr to graphic home
	glcd_msb = graph_home.BYTE1		
	glcd_lsb = graph_home.BYTE0
	glcd_cmd = GRAPH_HOME_SET
    gosub send_2
return

txt_home:	
	' Set Addr Ptr to graphic home
	glcd_msb = text_home.BYTE1		
	glcd_lsb = text_home.BYTE0
	glcd_cmd = TEXT_HOME_SET
        gosub send_2
return

'**********  End of PBP subroutines *********************
                                                    
'************* BEGIN MAIN PROGRAM *********************** 
'***Initialize and setup display headings,titles etc ****

main:
  gosub gl_init		'Initialize and clear display

  GoSub grf_txt_on        
  gl_x = 8	'Horiz pos
  gl_y = 8        'Vert pos
  For gl_i = 0 to 26
    LookUp gl_i,["Sanyo 240x64 using a T6963C"],gl_char
    GoSub put_char
    gl_x = gl_x + 8
  Next gl_i

  gl_x = 32	'Horiz pos
  gl_y = 24 'Vert pos
  For gl_i = 0 to 18
    LookUp gl_i,[" By COMPSys(c) 2003"],gl_char  
    GoSub put_char
    gl_x = gl_x + 8
  Next gl_i

  gl_x = 40	'Horiz pos
  gl_y = 40   'Vert pos
  For gl_i = 0 to 17
    LookUp gl_i,["Use GLiPIC2 v0.9.6"],gl_char  
    GoSub put_char
    gl_x = gl_x + 8
  Next gl_i
  
   gl_x = 72	'Horiz pos
  gl_y = 56   'Vert pos
  For gl_i = 0 to 10
    LookUp gl_i,["Waiting...."],gl_char  
    GoSub put_char
    gl_x = gl_x + 8
  Next gl_i 

'Terminal Subroutine
term:		'Assumes Home Position

'Setup variables
  gl_j=0
  gl_index=0
  gl_k=0
  buff_count = 32
  gl_delay = 10
  gl_addr = 0
  gl_x = 0
  gl_y = 0
  ptr_flag = DATA_WR       'or DATA_WR_INC 
  gl_idx = 120
  gl_idy = 0
  eep_addr = 0

'Get the initial parameters
get_param:
	serin2 in_pin,ser_baud,[wait ("*"),STR in_ctl\9]
    gosub grf_on
	for gl_index = 0 to 8
        Lookdown in_ctl[gl_index],["0123456789abcdef"],hex_nibble
	in_ctl[gl_index]=hex_nibble
	next
	gl_delay = (in_ctl[7]<<4) | in_ctl[8]
	buff_count = (in_ctl[1]<<4) | in_ctl[2]
	gl_addr.byte1 = (in_ctl[3] <<4) | in_ctl[4]
	gl_addr.byte0 = (in_ctl[5] <<4) | in_ctl[6]
    eep_addr = gl_addr
	gl_j=in_ctl[0]
	if gl_j  = 0 then lcd_fill 		'Display directly on LCD
    if gl_j  = 2 then lcd_fill 		'Display directly on LCD
	if gl_j  = 1 then eep_loop		'Load ext eeprom
	if gl_j =  4 then lcd_fill		'Display images loop
	if gl_j =  5 then eep_disp		'For Horiz bytes
	goto get_param 
eep_loop:
	 serin2 in_pin,ser_baud,5000,eep_done,[wait ("]"),STR in_array\buff_count] 	'Wait for ] and the load in_array

	 for gl_index = 0 to (buff_count-1)
                'Get char and place in array
	        Lookdown in_array[gl_index],["0123456789abcdef"],hex_nibble
		in_array[gl_index]=hex_nibble
	 next

	for gl_index = 0 to (buff_count-1) step 2
		gl_byte = (in_array[gl_index]<<4) | (in_array[gl_index+1])	'Form a hex number MSB-LSB
	        I2CWRITE sda,scl,ctl,eep_addr,[gl_byte]
	        pause 5
                eep_addr=eep_addr+1                
	next
goto eep_loop
eep_disp:
	'gosub grf_clr	
	gl_idx   = (buff_count*8) - 1	'Use for x limit (29 for 240x64)
	gl_idy   = gl_delay - 1	        'Use for y limit (64 for 240x64)
	gl_x     = 0
	gl_y     = 0
    gosub gl_gaddr
      ptr_flag = DATA_WR       'DATA_WR  or DATA_WR_INC	
	  for gl_y = 0 to gl_idy  '64 vert lines and 29 bytes across
	       for gl_x = 0 to gl_idx step 8		
	        	I2CREAD sda,scl,ctl,eep_addr,[gl_byte]
			gosub put_byte
			eep_addr = eep_addr + 1
	       next gl_x
	  next gl_y
goto get_param

lcd_fill:
gl_idx = 232
gl_idy = 0
gl_x   = 0
gl_y   = 0
eep_addr = 0

lcd_loop: 
	 serin2 in_pin,ser_baud,2000,get_param,[wait ("]"),STR in_array\buff_count] 	'Wait for ] and the load in_array
	 for gl_index = 0 to (buff_count - 1)
            gosub gl_find			'Get char and place in array
	        in_array[gl_index]=hex_nibble
	 next
	for gl_index = 0 to (buff_count - 1) step 2
		gl_byte = (in_array[gl_index]<<4) | (in_array[gl_index+1])	'Form a hex number MSB-LSB
		if gl_x > gl_idx  then
			gl_x = gl_idy
			gl_y = gl_y + 1
		endif
  		if gl_y > 63 then gl_y = 0
   	    gl_addr	= graph_home + (gl_y * GL_ROW) + (gl_x/font_size)
	    glcd_msb = gl_addr.BYTE1
	    glcd_lsb = gl_addr.BYTE0
	    glcd_cmd = ADDR_PTR_SET
	    gosub send_2
	    glcd_msb = gl_byte
	    glcd_cmd = ptr_flag
	    gosub send_1
	    gl_x = gl_x + 8
	next
goto lcd_loop

eep_done:
serout2 out_pin,ser_baud,["Last byte at: ",DEC eep_addr,10,13]
goto get_param

end
'******** END OF MAIN PROGRAM **************


